#!/usr/bin/python3

import os
import sys
from lxml import etree

if len(sys.argv) < 3:
    print('Invalid number of arguments', file=sys.stderr)
    sys.exit(1)

INTEL_NESTED_PATH = "/sys/module/kvm_intel/parameters/nested"
AMD_NESTED_PATH = "/sys/module/kvm_amd/parameters/nested"

def check_nested_virtualization():
    nested_path = None
    vendor = None
    if os.path.exists(INTEL_NESTED_PATH):
        nested_path = INTEL_NESTED_PATH
        vendor = "Intel"
    elif os.path.exists(AMD_NESTED_PATH):
        nested_path = AMD_NESTED_PATH
        vendor = "AMD"

    if not nested_path:
        return [False, vendor]

    with open(nested_path, 'r') as f:
        return [f.read().strip() == '1', vendor]

op = sys.argv[2]
if op == 'define':
    doc = etree.parse(sys.stdin)

    ostype = doc.xpath('/domain/os/type')
    if not ostype:
        print('Element /domain/os/type is mandatory', file=sys.stderr)
        sys.exit(1)
    ostype = ostype[0]

    if not ostype.xpath('boolean(@machine)'):
        # default to q35
        ostype.attrib['machine'] = 'q35'

    is_nested, vendor = check_nested_virtualization()

    ns = {'libosinfo': 'http://libosinfo.org/xmlns/libvirt/domain/1.0'}
    osidpath = '/domain/metadata/libosinfo:libosinfo/libosinfo:os/@id'
    osid = doc.xpath(osidpath, namespaces=ns)

    is_win = osid and 'microsoft.com/win' in osid[0]

    if is_win:
        domain = doc.xpath('/domain')[0]
        features = domain.xpath('./features')

        if features:
            features = features[0]
        else:
            features = etree.SubElement(domain, 'features')

        hyperv = features.xpath('./hyperv')

        if hyperv:
            hyperv = hyperv[0]
        else:
            hyperv = etree.SubElement(features, 'hyperv')

        if is_nested:
            if vendor != "AMD" and not hyperv.xpath('./evmcs'):
                etree.SubElement(hyperv, 'evmcs', state='on')

            stimer = hyperv.xpath("./stimer")

            if stimer:
                stimer = stimer[0]
            else:
                stimer = etree.SubElement(hyperv, 'stimer', state='on')

            if stimer.attrib['state'] == 'on' and not stimer.xpath('./direct'):
                etree.SubElement(stimer, 'direct', state='on')

        for f in ["frequencies", "tlbflush", "ipi"]:
            if not hyperv.xpath('./' + f):
                etree.SubElement(hyperv, f, state='on')

    print(etree.tostring(doc, encoding='unicode'))
